/* 
 * Copyright © 2015 Sebastian Kügler <sebas@kde.org>
 * Copyright © 2008-2011 Kristian Høgsberg
 * Copyright © 2010-2011 Intel Corporation
 * Copyright © 2012-2013 Collabora, Ltd.
 * 
 * Permission to use, copy, modify, distribute, and sell this
 * software and its documentation for any purpose is hereby granted
 * without fee, provided that the above copyright notice appear in
 * all copies and that both that copyright notice and this permission
 * notice appear in supporting documentation, and that the name of
 * the copyright holders not be used in advertising or publicity
 * pertaining to distribution of the software without specific,
 * written prior permission.  The copyright holders make no
 * representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied
 * warranty.
 * 
 * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS, IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
 * AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
 * ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF
 * THIS SOFTWARE.
 */

#ifndef ORG_KDE_KWIN_OUTPUTDEVICE_CLIENT_PROTOCOL_H
#define ORG_KDE_KWIN_OUTPUTDEVICE_CLIENT_PROTOCOL_H

#ifdef  __cplusplus
extern "C" {
#endif

#include <stdint.h>
#include <stddef.h>
#include "wayland-client.h"

struct wl_client;
struct wl_resource;

struct org_kde_kwin_outputdevice;

extern const struct wl_interface org_kde_kwin_outputdevice_interface;

#ifndef ORG_KDE_KWIN_OUTPUTDEVICE_SUBPIXEL_ENUM
#define ORG_KDE_KWIN_OUTPUTDEVICE_SUBPIXEL_ENUM
/**
 * org_kde_kwin_outputdevice_subpixel - subpixel geometry information
 * @ORG_KDE_KWIN_OUTPUTDEVICE_SUBPIXEL_UNKNOWN: (none)
 * @ORG_KDE_KWIN_OUTPUTDEVICE_SUBPIXEL_NONE: (none)
 * @ORG_KDE_KWIN_OUTPUTDEVICE_SUBPIXEL_HORIZONTAL_RGB: (none)
 * @ORG_KDE_KWIN_OUTPUTDEVICE_SUBPIXEL_HORIZONTAL_BGR: (none)
 * @ORG_KDE_KWIN_OUTPUTDEVICE_SUBPIXEL_VERTICAL_RGB: (none)
 * @ORG_KDE_KWIN_OUTPUTDEVICE_SUBPIXEL_VERTICAL_BGR: (none)
 *
 * This enumeration describes how the physical pixels on an output are
 * laid out.
 */
enum org_kde_kwin_outputdevice_subpixel {
	ORG_KDE_KWIN_OUTPUTDEVICE_SUBPIXEL_UNKNOWN = 0,
	ORG_KDE_KWIN_OUTPUTDEVICE_SUBPIXEL_NONE = 1,
	ORG_KDE_KWIN_OUTPUTDEVICE_SUBPIXEL_HORIZONTAL_RGB = 2,
	ORG_KDE_KWIN_OUTPUTDEVICE_SUBPIXEL_HORIZONTAL_BGR = 3,
	ORG_KDE_KWIN_OUTPUTDEVICE_SUBPIXEL_VERTICAL_RGB = 4,
	ORG_KDE_KWIN_OUTPUTDEVICE_SUBPIXEL_VERTICAL_BGR = 5,
};
#endif /* ORG_KDE_KWIN_OUTPUTDEVICE_SUBPIXEL_ENUM */

#ifndef ORG_KDE_KWIN_OUTPUTDEVICE_TRANSFORM_ENUM
#define ORG_KDE_KWIN_OUTPUTDEVICE_TRANSFORM_ENUM
/**
 * org_kde_kwin_outputdevice_transform - transform from framebuffer to
 *	output
 * @ORG_KDE_KWIN_OUTPUTDEVICE_TRANSFORM_NORMAL: (none)
 * @ORG_KDE_KWIN_OUTPUTDEVICE_TRANSFORM_90: (none)
 * @ORG_KDE_KWIN_OUTPUTDEVICE_TRANSFORM_180: (none)
 * @ORG_KDE_KWIN_OUTPUTDEVICE_TRANSFORM_270: (none)
 * @ORG_KDE_KWIN_OUTPUTDEVICE_TRANSFORM_FLIPPED: (none)
 * @ORG_KDE_KWIN_OUTPUTDEVICE_TRANSFORM_FLIPPED_90: (none)
 * @ORG_KDE_KWIN_OUTPUTDEVICE_TRANSFORM_FLIPPED_180: (none)
 * @ORG_KDE_KWIN_OUTPUTDEVICE_TRANSFORM_FLIPPED_270: (none)
 *
 * This describes the transform that a compositor will apply to a surface
 * to compensate for the rotation or mirroring of an output device.
 *
 * The flipped values correspond to an initial flip around a vertical axis
 * followed by rotation.
 *
 * The purpose is mainly to allow clients render accordingly and tell the
 * compositor, so that for fullscreen surfaces, the compositor will still
 * be able to scan out directly from client surfaces.
 */
enum org_kde_kwin_outputdevice_transform {
	ORG_KDE_KWIN_OUTPUTDEVICE_TRANSFORM_NORMAL = 0,
	ORG_KDE_KWIN_OUTPUTDEVICE_TRANSFORM_90 = 1,
	ORG_KDE_KWIN_OUTPUTDEVICE_TRANSFORM_180 = 2,
	ORG_KDE_KWIN_OUTPUTDEVICE_TRANSFORM_270 = 3,
	ORG_KDE_KWIN_OUTPUTDEVICE_TRANSFORM_FLIPPED = 4,
	ORG_KDE_KWIN_OUTPUTDEVICE_TRANSFORM_FLIPPED_90 = 5,
	ORG_KDE_KWIN_OUTPUTDEVICE_TRANSFORM_FLIPPED_180 = 6,
	ORG_KDE_KWIN_OUTPUTDEVICE_TRANSFORM_FLIPPED_270 = 7,
};
#endif /* ORG_KDE_KWIN_OUTPUTDEVICE_TRANSFORM_ENUM */

#ifndef ORG_KDE_KWIN_OUTPUTDEVICE_MODE_ENUM
#define ORG_KDE_KWIN_OUTPUTDEVICE_MODE_ENUM
/**
 * org_kde_kwin_outputdevice_mode - mode information
 * @ORG_KDE_KWIN_OUTPUTDEVICE_MODE_CURRENT: indicates this is the current
 *	mode
 * @ORG_KDE_KWIN_OUTPUTDEVICE_MODE_PREFERRED: indicates this is the
 *	preferred mode
 *
 * These flags describe properties of an output mode. They are used in
 * the flags bitfield of the mode event.
 */
enum org_kde_kwin_outputdevice_mode {
	ORG_KDE_KWIN_OUTPUTDEVICE_MODE_CURRENT = 0x1,
	ORG_KDE_KWIN_OUTPUTDEVICE_MODE_PREFERRED = 0x2,
};
#endif /* ORG_KDE_KWIN_OUTPUTDEVICE_MODE_ENUM */

#ifndef ORG_KDE_KWIN_OUTPUTDEVICE_ENABLEMENT_ENUM
#define ORG_KDE_KWIN_OUTPUTDEVICE_ENABLEMENT_ENUM
/**
 * org_kde_kwin_outputdevice_enablement - device is used to display
 *	content by the compositor
 * @ORG_KDE_KWIN_OUTPUTDEVICE_ENABLEMENT_DISABLED: (none)
 * @ORG_KDE_KWIN_OUTPUTDEVICE_ENABLEMENT_ENABLED: (none)
 *
 * Describes whether a device is enabled, i.e. device is used to display
 * content by the compositor. This wraps a boolean around an int to avoid a
 * boolean trap.
 */
enum org_kde_kwin_outputdevice_enablement {
	ORG_KDE_KWIN_OUTPUTDEVICE_ENABLEMENT_DISABLED = 0,
	ORG_KDE_KWIN_OUTPUTDEVICE_ENABLEMENT_ENABLED = 1,
};
#endif /* ORG_KDE_KWIN_OUTPUTDEVICE_ENABLEMENT_ENUM */

/**
 * org_kde_kwin_outputdevice - compositor output region
 * @geometry: properties of the output
 * @mode: advertise available modes for the output
 * @done: sent all information about output
 * @scale: output scaling properties
 * @edid: advertise EDID data for the output
 * @enabled: advertise available modes for the output
 * @uuid: A unique id for this outputdevice
 *
 * An outputdevice describes a physical monitor connected to the system.
 * outputdevice is similar to wl_output, but focuses on screen management.
 * The screen configuration client uses outputdevices to enlist connected
 * devices, even those that may currently not be used by the compositor as
 * wl_output. It sends configuration changes to the compositor through the
 * outputconfiguration interface, which applies the settings to the
 * hardware and signals changes the outputdevices accordingly. This object
 * is published as global during start up, or when a monitor is hotplugged.
 */
struct org_kde_kwin_outputdevice_listener {
	/**
	 * geometry - properties of the output
	 * @x: x position within the global compositor space
	 * @y: y position within the global compositor space
	 * @physical_width: width in millimeters of the output
	 * @physical_height: height in millimeters of the output
	 * @subpixel: subpixel orientation of the output
	 * @make: textual description of the manufacturer
	 * @model: textual description of the model
	 * @transform: transform that maps framebuffer to output
	 *
	 * The geometry event describes geometric properties of the
	 * output. The event is sent when binding to the output object and
	 * whenever any of the properties change.
	 */
	void (*geometry)(void *data,
			 struct org_kde_kwin_outputdevice *org_kde_kwin_outputdevice,
			 int32_t x,
			 int32_t y,
			 int32_t physical_width,
			 int32_t physical_height,
			 int32_t subpixel,
			 const char *make,
			 const char *model,
			 int32_t transform);
	/**
	 * mode - advertise available modes for the output
	 * @flags: bitfield of mode flags
	 * @width: width of the mode in hardware units
	 * @height: height of the mode in hardware units
	 * @refresh: vertical refresh rate in mHz
	 * @mode_id: Per outputdevice unique id to identify a mode
	 *
	 * The mode event describes an available mode for the output.
	 *
	 * The event is sent when binding to the output object and there
	 * will always be one mode, the current mode. The event is sent
	 * again if an output changes mode, for the mode that is now
	 * current. In other words, the current mode is always the last
	 * mode that was received with the current flag set.
	 *
	 * The size of a mode is given in physical hardware units of the
	 * output device. This is not necessarily the same as the output
	 * size in the global compositor space. For instance, the output
	 * may be scaled, as described in org_kde_kwin_outputdevice.scale,
	 * or transformed , as described in
	 * org_kde_kwin_outputdevice.transform.
	 *
	 * The id can be used to refer to a mode when calling set_mode on a
	 * org_kde_kwin_outputonfiguration.
	 */
	void (*mode)(void *data,
		     struct org_kde_kwin_outputdevice *org_kde_kwin_outputdevice,
		     uint32_t flags,
		     int32_t width,
		     int32_t height,
		     int32_t refresh,
		     int32_t mode_id);
	/**
	 * done - sent all information about output
	 *
	 * This event is sent after all other properties has been sent
	 * after binding to the output object and after any other property
	 * changes done after that. This allows changes to the output
	 * properties to be seen as atomic, even if they happen via
	 * multiple events.
	 */
	void (*done)(void *data,
		     struct org_kde_kwin_outputdevice *org_kde_kwin_outputdevice);
	/**
	 * scale - output scaling properties
	 * @factor: scaling factor of output
	 *
	 * This event contains scaling geometry information that is not
	 * in the geometry event. It may be sent after binding the output
	 * object or if the output scale changes later. If it is not sent,
	 * the client should assume a scale of 1.
	 *
	 * A scale larger than 1 means that the compositor will
	 * automatically scale surface buffers by this amount when
	 * rendering. This is used for very high resolution displays where
	 * applications rendering at the native resolution would be too
	 * small to be legible.
	 *
	 * It is intended that scaling aware clients track the current
	 * output of a surface, and if it is on a scaled output it should
	 * use wl_surface.set_buffer_scale with the scale of the output.
	 * That way the compositor can avoid scaling the surface, and the
	 * client can supply a higher detail image.
	 */
	void (*scale)(void *data,
		      struct org_kde_kwin_outputdevice *org_kde_kwin_outputdevice,
		      int32_t factor);
	/**
	 * edid - advertise EDID data for the output
	 * @raw: base64-encoded EDID string
	 *
	 * The edid event encapsulates the EDID data for the
	 * outputdevice.
	 *
	 * The event is sent when binding to the output object. The EDID
	 * data may be empty, in which case this event is fired, anyway. If
	 * the EDID information is empty, you can fall back to the name et
	 * al. properties of the outputdevice.
	 */
	void (*edid)(void *data,
		     struct org_kde_kwin_outputdevice *org_kde_kwin_outputdevice,
		     const char *raw);
	/**
	 * enabled - advertise available modes for the output
	 * @enabled: output is enabled in the wayland server
	 *
	 * The enabled event notifies whether this output is currently
	 * enabled and used for displaying content the wayland server. The
	 * event is sent when binding to the output object, and when an
	 * output is enabled or disabled.
	 */
	void (*enabled)(void *data,
			struct org_kde_kwin_outputdevice *org_kde_kwin_outputdevice,
			int32_t enabled);
	/**
	 * uuid - A unique id for this outputdevice
	 * @uuid: Output devices ID
	 *
	 * The uuid can be used to identify the output. It's controlled
	 * by the server entirely. The compositor should make sure the uuid
	 * is persistant across restarts. An empty uuid is considered
	 * invalid.
	 */
	void (*uuid)(void *data,
		     struct org_kde_kwin_outputdevice *org_kde_kwin_outputdevice,
		     const char *uuid);
};

static inline int
org_kde_kwin_outputdevice_add_listener(struct org_kde_kwin_outputdevice *org_kde_kwin_outputdevice,
				       const struct org_kde_kwin_outputdevice_listener *listener, void *data)
{
	return wl_proxy_add_listener((struct wl_proxy *) org_kde_kwin_outputdevice,
				     (void (**)(void)) listener, data);
}

static inline void
org_kde_kwin_outputdevice_set_user_data(struct org_kde_kwin_outputdevice *org_kde_kwin_outputdevice, void *user_data)
{
	wl_proxy_set_user_data((struct wl_proxy *) org_kde_kwin_outputdevice, user_data);
}

static inline void *
org_kde_kwin_outputdevice_get_user_data(struct org_kde_kwin_outputdevice *org_kde_kwin_outputdevice)
{
	return wl_proxy_get_user_data((struct wl_proxy *) org_kde_kwin_outputdevice);
}

static inline void
org_kde_kwin_outputdevice_destroy(struct org_kde_kwin_outputdevice *org_kde_kwin_outputdevice)
{
	wl_proxy_destroy((struct wl_proxy *) org_kde_kwin_outputdevice);
}

#ifdef  __cplusplus
}
#endif

#endif
