/* 
 * Copyright (C) 2015 Martin Gräßlin
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef FAKE_INPUT_SERVER_PROTOCOL_H
#define FAKE_INPUT_SERVER_PROTOCOL_H

#ifdef  __cplusplus
extern "C" {
#endif

#include <stdint.h>
#include <stddef.h>
#include "wayland-server.h"

struct wl_client;
struct wl_resource;

struct org_kde_kwin_fake_input;

extern const struct wl_interface org_kde_kwin_fake_input_interface;

/**
 * org_kde_kwin_fake_input - Fake input manager
 * @authenticate: Information why the client wants to use the interface
 * @pointer_motion: (none)
 * @button: (none)
 * @axis: (none)
 *
 * This interface allows other processes to provide fake input events.
 * Purpose is on the one hand side to provide testing facilities like XTest
 * on X11. But also to support use case like kdeconnect's mouse pad
 * interface.
 *
 * A compositor should not trust the input received from this interface.
 * Clients should not expect that the compositor honors the requests from
 * this interface.
 */
struct org_kde_kwin_fake_input_interface {
	/**
	 * authenticate - Information why the client wants to use the
	 *	interface
	 * @application: (none)
	 * @reason: (none)
	 *
	 * A client should use this request to tell the compositor why it
	 * wants to use this interface. The compositor might use the
	 * information to decide whether it wants to grant the request. The
	 * data might also be passed to the user to decide whether the
	 * application should get granted access to this very privileged
	 * interface.
	 */
	void (*authenticate)(struct wl_client *client,
			     struct wl_resource *resource,
			     const char *application,
			     const char *reason);
	/**
	 * pointer_motion - (none)
	 * @delta_x: (none)
	 * @delta_y: (none)
	 */
	void (*pointer_motion)(struct wl_client *client,
			       struct wl_resource *resource,
			       wl_fixed_t delta_x,
			       wl_fixed_t delta_y);
	/**
	 * button - (none)
	 * @button: (none)
	 * @state: (none)
	 */
	void (*button)(struct wl_client *client,
		       struct wl_resource *resource,
		       uint32_t button,
		       uint32_t state);
	/**
	 * axis - (none)
	 * @axis: (none)
	 * @value: (none)
	 */
	void (*axis)(struct wl_client *client,
		     struct wl_resource *resource,
		     uint32_t axis,
		     wl_fixed_t value);
};


#ifdef  __cplusplus
}
#endif

#endif
