/* 
 * Copyright (C) 2015 Martin Gräßlin
 * Copyright (C) 2015 Marco Martin
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef SLIDE_SERVER_PROTOCOL_H
#define SLIDE_SERVER_PROTOCOL_H

#ifdef  __cplusplus
extern "C" {
#endif

#include <stdint.h>
#include <stddef.h>
#include "wayland-server.h"

struct wl_client;
struct wl_resource;

struct org_kde_kwin_slide_manager;
struct org_kde_kwin_slide;

extern const struct wl_interface org_kde_kwin_slide_manager_interface;
extern const struct wl_interface org_kde_kwin_slide_interface;

struct org_kde_kwin_slide_manager_interface {
	/**
	 * create - (none)
	 * @id: (none)
	 * @surface: (none)
	 */
	void (*create)(struct wl_client *client,
		       struct wl_resource *resource,
		       uint32_t id,
		       struct wl_resource *surface);
	/**
	 * unset - (none)
	 * @surface: (none)
	 */
	void (*unset)(struct wl_client *client,
		      struct wl_resource *resource,
		      struct wl_resource *surface);
};


#ifndef ORG_KDE_KWIN_SLIDE_LOCATION_ENUM
#define ORG_KDE_KWIN_SLIDE_LOCATION_ENUM
enum org_kde_kwin_slide_location {
	ORG_KDE_KWIN_SLIDE_LOCATION_LEFT = 0,
	ORG_KDE_KWIN_SLIDE_LOCATION_TOP = 1,
	ORG_KDE_KWIN_SLIDE_LOCATION_RIGHT = 2,
	ORG_KDE_KWIN_SLIDE_LOCATION_BOTTOM = 3,
};
#endif /* ORG_KDE_KWIN_SLIDE_LOCATION_ENUM */

/**
 * org_kde_kwin_slide - slide a surface from a location to another
 * @commit: (none)
 * @set_location: (none)
 * @set_offset: (none)
 * @release: release the slide object
 *
 * Ask the compositor to move the surface from a location to another with
 * a slide animation.
 *
 * The from argument provides a clue about where the slide animation
 * begins, offset is the distance from screen edge to begin the animation.
 */
struct org_kde_kwin_slide_interface {
	/**
	 * commit - (none)
	 */
	void (*commit)(struct wl_client *client,
		       struct wl_resource *resource);
	/**
	 * set_location - (none)
	 * @location: (none)
	 */
	void (*set_location)(struct wl_client *client,
			     struct wl_resource *resource,
			     uint32_t location);
	/**
	 * set_offset - (none)
	 * @offset: (none)
	 */
	void (*set_offset)(struct wl_client *client,
			   struct wl_resource *resource,
			   int32_t offset);
	/**
	 * release - release the slide object
	 *
	 * 
	 */
	void (*release)(struct wl_client *client,
			struct wl_resource *resource);
};


#ifdef  __cplusplus
}
#endif

#endif
